# # 程序信息
# # 作者：格蕾特（グレーテ）
# # 描述：GoldenDict-ng 外置程序，可实现添加笔记、记录查询信息等功能，参见 https://forum.freemdict.com/t/topic/34974
# # 著作权声明：适用知识共享许可协议 CC BY-NC-SA 4.0（署名—非商业性使用—相同方式共享）https://creativecommons.org/licenses/by-nc-sa/4.0/deed.zh-hans
# # 目前版本：3.0
# # 历史版本：
# # 3.0    2025年 1月 9日    修改功能：允许同一条目在多个分组下创建笔记
# #                          增加功能：允许用户往笔记内添加图片
# #                                    允许用户通过选择文本进行拖动；绑定 Ctrl+B/I/U 快捷键
# # 2.0    2024年12月31日    增加功能：笔记本分组
# # 1.0    2024年12月18日    增加功能：添加笔记、记录查询信息

import sys
import os
import re
import tkinter as tk
from tkinter import simpledialog


# 设置输出编码为 UTF-8
sys.stdout.reconfigure(encoding='utf-8')

# 配置功能开关
ENABLE_NOTE = True  # 控制笔记功能
ENABLE_QUERY_COUNT = False  # 控制查询次数记录功能

# 文件夹路径配置
NOTE_FOLDER = r'C:\MySoftware\GoldenDict\note'
DEFAULT_NOTE_FILE = 'GoldenDict-MyNote.txt'
QUERY_COUNT_FILE = 'GoldenDict-Query.txt'


QUERY_COUNT_FILE = os.path.join(NOTE_FOLDER, QUERY_COUNT_FILE)

# 获取查询词
if len(sys.argv) < 2:
    print("<html><body>Error: No word provided.</body></html>")
    sys.exit(1)

word = sys.argv[1]

# 判断查询词是否包含 edit=true
is_edit_mode = '?edit=true' in word

# 去除查询词中的 ?edit=true 部分，只在末尾出现时去除
if word.endswith("?edit=true"):
    word = word[:-10]  # 去掉末尾的 ?edit=true

# 根据查询词中的 = 拆分，判断是否指定编辑笔记分组
is_multiple_group = word.split('=')
word = is_multiple_group[0]
selected_group = []
if len(is_multiple_group) == 2:
    selected_group = is_multiple_group[1]

# 检查文件夹是否存在
if not os.path.exists(NOTE_FOLDER):
    print(f"<html><body>Error: Note folder not found: {NOTE_FOLDER}</body></html>")
    sys.exit(1)


if ENABLE_QUERY_COUNT:
    from datetime import datetime, timezone, timedelta
    import pytz  # 用于处理时区
    # 获取用户本地时区
    local_tz = datetime.now().astimezone().tzinfo


# 记录查询次数
if ENABLE_QUERY_COUNT:
    query_data = {}

    if os.path.exists(QUERY_COUNT_FILE):
        with open(QUERY_COUNT_FILE, 'r', encoding='utf-8') as file:
            for line in file:
                parts = line.strip().split('\t')
                if len(parts) == 4:
                    q_word, count, first_time, last_time = parts
                    query_data[q_word] = {
                        'count': int(count),
                        'first_time': first_time,
                        'last_time': last_time
                    }

    now = datetime.now(local_tz).replace(microsecond=0)  # now 转换为 datetime 对象
    
    if word in query_data:
        prev_last_time = query_data[word]['last_time']
        # 计算上次查询时间与当前时间的差值
        last_time_obj = datetime.fromisoformat(prev_last_time).astimezone(local_tz)
        time_diff = now - last_time_obj

        # 如果上次查询时间与当前时间的差值小于10分钟，不增加查询次数
        if time_diff < timedelta(minutes=10):
            query_output = ""  # 不计入查询
        else:
            query_data[word]['count'] += 1
            query_data[word]['last_time'] = now.isoformat()  # 保存为 ISO 格式字符串
    else:
        prev_last_time = "N/A"
        query_data[word] = {
            'count': 1,
            'first_time': now.isoformat(),  # 保存为 ISO 格式字符串
            'last_time': now.isoformat()  # 保存为 ISO 格式字符串
        }

    # 更新文件
    with open(QUERY_COUNT_FILE, 'w', encoding='utf-8') as file:
        for q_word, data in query_data.items():
            file.write(f"{q_word}\t{data['count']}\t{data['first_time']}\t{data['last_time']}\n")

# 获取查询次数输出
if ENABLE_QUERY_COUNT and word in query_data:
    data = query_data[word]
    first_time = datetime.fromisoformat(data['first_time']).astimezone(local_tz)
    last_time = datetime.fromisoformat(data['last_time']).astimezone(local_tz)
    now = datetime.now(local_tz)

    def format_time_difference(time):
        diff = now - time
        seconds = diff.total_seconds()
        if seconds < 86400:
            if seconds < 60:
                return f"{int(seconds)} seconds ago"
            elif seconds < 3600:
                return f"{seconds / 60:.1f} minutes ago"
            else:
                return f"{seconds / 3600:.1f} hours ago"
        else:
            return time.strftime('%Y-%m-%dT%H:%M:%S%z')

    first_time_display = format_time_difference(first_time)

    if prev_last_time != "N/A":
        second_to_last_time = datetime.fromisoformat(prev_last_time).astimezone(local_tz)
        last_time_display = format_time_difference(second_to_last_time)
    else:
        last_time_display = "N/A"

    query_output = (
        f"<b>{word}</b> queried <b>{data['count']}</b> times, "
        f"first query time is <b>{first_time_display}</b>, "
        f"last query time is: <b>{last_time_display}</b>."
    )
else:
    query_output = ""
    

# 遍历文件夹中的笔记文件
if ENABLE_NOTE:
    if selected_group:
        note_files = [f for f in os.listdir(NOTE_FOLDER) if f.startswith('GoldenDict-MyNote') and f.endswith('.txt')]
        
        # 根据笔记分组还原笔记位置
        selected_note_file = f"GoldenDict-MyNote-{selected_group}.txt"
        if selected_note_file == "GoldenDict-MyNote-[Default].txt":
            selected_note_file = DEFAULT_NOTE_FILE
        
        # 配合无选定组情况的代码构建 all_notes 数组
        all_notes = {}
        with open(os.path.join(NOTE_FOLDER, selected_note_file), 'r', encoding='utf-8') as f:
            current_note_content = f.read()
            all_notes[selected_note_file] = current_note_content
        
        current_note_content = all_notes.get(selected_note_file, '')

        # 构建正则表达式
        pattern = rf"{re.escape(word)}\n*<font size=5>{re.escape(word)}</font>\n*<p class=\"senselevel-1\">(.*?)</p>\n*</>"

        # 查找匹配内容
        match = re.search(pattern, current_note_content, re.DOTALL)
        
        # 判断是否匹配到内容
        if match:
            # 提取匹配的内容
            result = match.group(1).strip()
        else:
            # 查询词不存在，设置为错误消息
            result = f"No note found for the word: {word}"
                
    else:
        note_files = [f for f in os.listdir(NOTE_FOLDER) if f.startswith('GoldenDict-MyNote') and f.endswith('.txt')]

        # 设置默认笔记文件
        selected_note_file = DEFAULT_NOTE_FILE

        # 读取所有笔记内容
        def read_all_notes():
            notes = {}
            for file in note_files:
                file_path = os.path.join(NOTE_FOLDER, file)
                if os.path.exists(file_path):
                    with open(file_path, 'r', encoding='utf-8') as f:
                        notes[file] = f.read()
            return notes

        # 获取当前笔记内容
        all_notes = read_all_notes()
        current_note_content = all_notes.get(selected_note_file, '')

        # 查找笔记
        pattern = rf"{re.escape(word)}\n*<font size=5>{re.escape(word)}</font>\n*<p class=\"senselevel-1\">(.*?)</p>\n*</>"
        match = re.search(pattern, current_note_content, re.DOTALL)

# 在所有分组中查找匹配的笔记
if ENABLE_NOTE:
    if selected_group:
        pass
    else:
        def find_notes_in_all_groups(word):
            matches = []
            for file, content in all_notes.items():
                pattern = rf"{re.escape(word)}\n*<font size=5>{re.escape(word)}</font>\n*<p class=\"senselevel-1\">(.*?)</p>\n*</>"
                match = re.search(pattern, content, re.DOTALL)
                if match:
                    matches.append((match.group(1).strip(), file))  # 收集匹配内容和文件名
            return matches

        # 从所有分组中查找笔记
        results = find_notes_in_all_groups(word)

if ENABLE_NOTE:
    if selected_group:
        result_flag = True
    else:
        result_flag = True
        len_results = len(results)
        all_group_names = []
        if not results: # 未找到笔记
            result_flag = False
            result = f"No note found for the word: {word}"
        elif len_results == 1: # 笔记仅见于一个分组
            result, selected_note_file = results[0]
            group_name = selected_note_file.replace('GoldenDict-MyNote-', '').replace('.txt', '') if selected_note_file != DEFAULT_NOTE_FILE else "[Default]"
            all_group_names.append(group_name)
        else:   # 笔记见于多个分组
            formatted_results = ""
            for content, file in results:
                group_name = file.replace('GoldenDict-MyNote-', '').replace('.txt', '') if file != DEFAULT_NOTE_FILE else "[Default]"
                formatted_results += f"<b>Group: {group_name}</b><br/>{content}<br/><hr/>"
                all_group_names.append(group_name)
            result = formatted_results



# 保存新内容并更新分组
if ENABLE_NOTE:
    def save_content(new_content, new_group):
        
        global all_notes, selected_note_file, match, selected_group
        current_note_content = all_notes.get(selected_note_file, '')
        match = re.search(pattern, current_note_content, re.DOTALL)
        
        # 将换行符前补充 <br/> 标签，但保留原来的换行符
        new_content = new_content.replace('\n', '<br/>\n')
        
        # 保存前的文件名，保留原文件作为当前文件
        current_note_file = selected_note_file
        selected_note_file = f"GoldenDict-MyNote-{new_group}.txt"
        if selected_note_file == "GoldenDict-MyNote-[Default].txt":
            selected_note_file = DEFAULT_NOTE_FILE
        
        # 如果未修改笔记分组
        if current_note_file == selected_note_file:
            # 如果笔记为空，则删除该词条
            if new_content.strip() == "":
                # 如果词条存在，删除该词条
                if match:
                    old_entry = match.group(0)
                    current_note_content = current_note_content.replace(f"{old_entry}\n", "")  # 删除该条目
                    current_note_content = current_note_content.replace(f"\n{old_entry}", "")  # 删除该条目
            else:
                # 如果词条存在，替换旧内容
                if match:
                    old_entry = match.group(0)
                    new_entry = f"{word}\n<font size=5>{word}</font>\n<p class=\"senselevel-1\">{new_content}</p>\n</>"
                    current_note_content = current_note_content.replace(old_entry, new_entry)
                else:
                    # 如果词条不存在，按正确格式追加新内容
                    new_entry = f"{word}\n<font size=5>{word}</font>\n<p class=\"senselevel-1\">{new_content}</p>\n</>"
                    current_note_content += f"\n{new_entry}"
            
            with open(os.path.join(NOTE_FOLDER, selected_note_file), 'w', encoding='utf-8') as f:
                f.write(current_note_content)
        # 如果修改了笔记分组
        else:
            # 如果找到匹配的笔记，先从原分组删除
            # if current_note_file:
                # old_note_content = all_notes.get(current_note_file, '')
                # if match:
                    # old_entry = match.group(0)
                    # old_note_content = old_note_content.replace(f"{old_entry}\n", "")
                    # old_note_content = old_note_content.replace(f"\n{old_entry}", "")
                    # all_notes[current_note_file] = old_note_content
                    # with open(os.path.join(NOTE_FOLDER, current_note_file), 'w', encoding='utf-8') as f:
                        # f.write(old_note_content)

            # 保存新内容到新分组
            new_note_content = all_notes.get(selected_note_file, '')
            if new_content.strip():
                new_entry = f"{word}\n<font size=5>{word}</font>\n<p class=\"senselevel-1\">{new_content}</p>\n</>"
                new_note_content += f"\n{new_entry}"
                all_notes[selected_note_file] = new_note_content

            # 保存到文件
            with open(os.path.join(NOTE_FOLDER, selected_note_file), 'w', encoding='utf-8') as f:
                f.write(new_note_content)
        
        

# 定义编辑界面
if ENABLE_NOTE:
    def edit_interface():
        global selected_note_file
        root = tk.Tk()
        root.title(f"Edit Note for '{word}'")

        # 创建框架
        frame = tk.Frame(root)
        frame.pack(fill=tk.BOTH, expand=True, padx=10, pady=10)

        # 标签
        tk.Label(frame, text=f"Current Note for '{word}':").pack(pady=5)

        # 文本区域
        text_area = tk.Text(frame, wrap=tk.WORD, undo=True)
        text_area.pack(fill=tk.BOTH, expand=True)
        if result_flag:
            text_area.insert("1.0", re.sub(r'<br\s*/?>', '', result))
        text_area.focus_set()
        # 使用正则表达式替换 <br>, <br/> 和 <br />


        # 保存按钮
        def on_save():
            new_content = text_area.get("1.0", "end").strip()
            # 优先使用用户输入的新分组名称，否则使用下拉菜单中选择的分组
            new_group = group_entry.get().strip() or group_var.get().strip()
            
            # 在这里，我们确保正确处理默认分组的文件名
            if new_group == DEFAULT_NOTE_FILE.replace('GoldenDict-MyNote-', '').replace('.txt', ''):
                new_group = None  # 这表示使用默认分组文件
            
            save_content(new_content, new_group)
            
            # 关闭 Tkinter 窗口
            root.destroy()
            
            print('<script>window.location.replace("' + word + '");</script>')

        # 绑定 Ctrl + S 快捷键触发保存
        root.bind('<Control-s>', lambda event: on_save())

        # 绑定 Ctrl + Z 快捷键触发撤销
        def on_undo(event=None):
            try:
                text_area.edit_undo()  # 调用 Text 小部件的撤销方法
            except tk.TclError:
                pass  # 如果没有更多可撤销的操作，忽略错误

        root.bind('<Control-z>', on_undo)  # 绑定 Ctrl+Z 快捷键


        
        # 绑定窗口关闭事件
        def on_close():
            # 关闭 Tkinter 窗口
            root.destroy()
            
            # 当用户关闭窗口时，不执行保存操作
            print('<script>window.location.replace("' + word + '");</script>')
                
        root.protocol("WM_DELETE_WINDOW", on_close)  # 绑定窗口关闭按钮
        

        # 以下是 B, I, U, S 按钮的功能实现

        # 在选中的文本两端插入 HTML 标签，如果没有选中文本则插入标签
        def insert_tag(tag):
            start_idx = text_area.index(tk.SEL_FIRST) if text_area.tag_ranges(tk.SEL) else text_area.index(tk.INSERT)
            end_idx = text_area.index(tk.SEL_LAST) if text_area.tag_ranges(tk.SEL) else start_idx

            selected_text = text_area.get(start_idx, end_idx)

            # 如果有选中文本
            if selected_text:
                text_area.delete(start_idx, end_idx)
                text_area.insert(start_idx, f"<{tag}>{selected_text}</{tag}>")
            else:
                text_area.insert(start_idx, f"<{tag}></{tag}>")
                text_area.mark_set(tk.INSERT, start_idx)

        # 添加 B 按钮（粗体）
        def add_bold():
            insert_tag('b')

        # 添加 I 按钮（斜体）
        def add_italic():
            insert_tag('i')

        # 添加 U 按钮（下划线）
        def add_underline():
            insert_tag('u')

        # 添加 S 按钮（删除线）
        def add_strike():
            insert_tag('s')
            
        # 绑定 Ctrl + B/I/U 快捷键触发保存
        root.bind('<Control-b>', lambda event: add_bold())
        root.bind('<Control-i>', lambda event: add_italic())
        root.bind('<Control-u>', lambda event: add_underline())

        # 添加 专名 按钮（带下划线的文本）
        def add_proper_name():
            start_idx = text_area.index(tk.SEL_FIRST) if text_area.tag_ranges(tk.SEL) else text_area.index(tk.INSERT)
            end_idx = text_area.index(tk.SEL_LAST) if text_area.tag_ranges(tk.SEL) else start_idx

            selected_text = text_area.get(start_idx, end_idx)

            # 如果有选中文本
            if selected_text:
                text_area.delete(start_idx, end_idx)
                text_area.insert(start_idx, f'<span style="border-bottom: 1px solid; margin: 0 1px;">{selected_text}</span>')
            else:
                text_area.insert(start_idx, '<span style="border-bottom: 1px solid; margin: 0 1px;"></span>')
                text_area.mark_set(tk.INSERT, start_idx)

        # 添加 书名 按钮（带波浪下划线的文本）
        def add_book_title():
            start_idx = text_area.index(tk.SEL_FIRST) if text_area.tag_ranges(tk.SEL) else text_area.index(tk.INSERT)
            end_idx = text_area.index(tk.SEL_LAST) if text_area.tag_ranges(tk.SEL) else start_idx

            selected_text = text_area.get(start_idx, end_idx)

            # 如果有选中文本
            if selected_text:
                text_area.delete(start_idx, end_idx)
                text_area.insert(start_idx, f'<span style="text-decoration: underline; text-decoration-style: wavy; text-decoration-thickness: 1px; text-underline-offset: 3px; margin: 0 1px;">{selected_text}</span>')
            else:
                text_area.insert(start_idx, '<span style="text-decoration: underline; text-decoration-style: wavy; text-decoration-thickness: 1px; text-underline-offset: 3px; margin: 0 1px;"></span>')
                text_area.mark_set(tk.INSERT, start_idx)

        # 添加 href 按钮（超链接）
        def add_href():
            start_idx = text_area.index(tk.SEL_FIRST) if text_area.tag_ranges(tk.SEL) else text_area.index(tk.INSERT)
            end_idx = text_area.index(tk.SEL_LAST) if text_area.tag_ranges(tk.SEL) else start_idx

            selected_text = text_area.get(start_idx, end_idx)

            # 如果有选中文本
            if selected_text:
                selected_text = selected_text.strip()  # 去除多余的空白字符
                encoded_text = selected_text.replace(" ", "%20")    # 将空格替换为 %20
        
                # 将选中的文本插入为超链接格式
                text_area.delete(start_idx, end_idx)
                text_area.insert(start_idx, f'<a href="entry://{encoded_text}#section">{selected_text}</a>')
            else:
                text_area.insert(start_idx, '<a href="entry://EXAMPLE#section">EXAMPLE</a>')
                text_area.mark_set(tk.INSERT, start_idx)

        
        # 允许用户通过选择文本进行拖动
        content = ""  # 存储剪切的内容
        draging = False
        ctrl = False
        sel = None  # 这里定义 sel 变量

        def ctrlup(e):
            nonlocal ctrl
            if e.keycode == 17:
                ctrl = False

        def ctrldown(e):
            nonlocal ctrl
            if e.keycode == 17:
                ctrl = True

        def handledown(e):
            nonlocal content, sel
            try:
                if insel(e):
                    content = text_area.get("sel.first", "sel.last")
                    sel = (text_area.index("sel.first"), text_area.index("sel.last"))  # 保存选择的区域
                    return "break"
            except:
                content = ""

        def handledrag(e):
            nonlocal draging, content
            if content:
                draging = True
                text_area.mark_set("insert", "@%d,%d" % (e.x, e.y))
                return "break"

        def handleup(e):
            nonlocal draging, content, sel
            if draging:
                if not ctrl:
                    # 删除被选中的内容
                    text_area.delete(*sel)
                text_area.insert("insert", content)  # 插入剪切的内容
                draging = False
                content = ""
            else:
                if content:
                    text_area.tag_remove("sel", "1.0", "end")
                    content = ""
                    text_area.mark_set("insert", "@%d,%d" % (e.x, e.y))

        def insel(e):
            try:
                y1, x1 = map(int, text_area.index("sel.first").split("."))
                y2, x2 = map(int, text_area.index("@%d,%d" % (e.x, e.y)).split("."))
                y3, x3 = map(int, text_area.index("sel.last").split("."))
                if y3 > y2 > y1:
                    return 1
                if y2 == y1 and x2 > x1:
                    return 1
                elif y2 == y3 and x2 < x3:
                    return 1
            except:
                pass

        # 绑定鼠标拖放事件
        text_area.bind("<KeyPress>", ctrldown, 1)
        text_area.bind("<KeyRelease>", ctrlup, 1)
        text_area.bind("<B1-Motion>", handledrag, 1)
        text_area.bind("<Button-1>", handledown, 1)
        text_area.bind("<ButtonRelease-1>", handleup, 1)
        
        
        # 添加图片并将图片保存到 NOTE_FOLDER 目录下的 images 子目录
        def add_image():
            from tkinter import filedialog
            import shutil

            # 打开文件选择对话框
            image_path = filedialog.askopenfilename(
                title="Select an Image",
                filetypes=[("Image Files", "*.png;*.jpg;*.jpeg;*.gif;*.bmp")]
            )

            if image_path:
                # 确定目标路径
                target_dir = os.path.join(NOTE_FOLDER, "images")  # 图片存储目录
                os.makedirs(target_dir, exist_ok=True)  # 确保目录存在

                # 获取图片文件扩展名
                _, file_extension = os.path.splitext(image_path)

                # 基础文件名：使用当前词条名称
                base_name = word.replace(" ", "_")  # 替换空格以生成有效的文件名
                counter = 1

                # 确保文件名唯一
                while True:
                    new_image_name = f"{base_name}_{counter}{file_extension}"
                    target_path = os.path.join(target_dir, new_image_name)
                    if not os.path.exists(target_path):
                        break  # 文件名不存在，可以使用
                    counter += 1

                # 复制图片到目标目录
                shutil.copy(image_path, target_path)

                # 在文本区域插入图片的 HTML 标签
                start_idx = text_area.index(tk.INSERT)  # 插入位置
                text_area.insert(start_idx, f'<img src="file://{target_path}" alt="Image" style="max-width:100%; height:auto;">\n')

        
        
        # 创建按钮框架，用于放置按钮
        button_frame = tk.Frame(root)
        button_frame.pack(fill=tk.X, padx=10, pady=(0, 10))


        # 创建 B, I, U, S, 专名, 书名, href 按钮
        tk.Button(button_frame, text="B", command=add_bold).pack(side=tk.LEFT, padx=5, pady=5)
        tk.Button(button_frame, text="I", command=add_italic).pack(side=tk.LEFT, padx=5, pady=5)
        tk.Button(button_frame, text="U", command=add_underline).pack(side=tk.LEFT, padx=5, pady=5)
        tk.Button(button_frame, text="S", command=add_strike).pack(side=tk.LEFT, padx=5, pady=5)
        tk.Button(button_frame, text="🔗", command=add_href).pack(side=tk.LEFT, padx=5, pady=5)
        tk.Button(button_frame, text="🖋", command=add_proper_name).pack(side=tk.LEFT, padx=5, pady=5)
        tk.Button(button_frame, text="📖", command=add_book_title).pack(side=tk.LEFT, padx=5, pady=5)
        # 创建 "插入图片" 按钮
        tk.Button(button_frame, text="🖼", command=add_image).pack(side=tk.LEFT, padx=5, pady=5)

        # 保存按钮
        tk.Button(button_frame, text="💾", command=on_save).pack(side=tk.RIGHT, padx=5, pady=5)


        # 创建另一个框架，用于放置下拉菜单和输入框
        frame_dropdown = tk.Frame(root)
        frame_dropdown.pack(fill=tk.X, padx=10, pady=(0, 15))


        # 下拉菜单初始化，横向排列
        existing_groups = [f.replace('GoldenDict-MyNote-', '').replace('.txt', '') for f in note_files if f != DEFAULT_NOTE_FILE]

        # 包括默认分组 '默认' 或相应的 group_name
        existing_groups.insert(0, '[Default]')  # 显示为 [Default]，但选择时会映射回默认文件

        # 设置 group_var 初始值
        if selected_note_file in note_files or selected_note_file == DEFAULT_NOTE_FILE:
            if selected_note_file == DEFAULT_NOTE_FILE:
                current_group = '[Default]'
            else:
                current_group = selected_note_file.replace('GoldenDict-MyNote-', '').replace('.txt', '')
        else:
            # 如果不存在，则回退到默认组
            current_group = '[Default]'
            selected_note_file = DEFAULT_NOTE_FILE

        group_var = tk.StringVar(value=current_group)


        # 下拉菜单部分
        tk.Label(frame_dropdown, text="Select an existing group:").pack(side=tk.LEFT, padx=5)
        group_dropdown = tk.OptionMenu(frame_dropdown, group_var, *existing_groups)
        group_dropdown.pack(side=tk.LEFT, padx=5)

        # 分组输入框（可选新分组），横向排列
        tk.Label(frame_dropdown, text="Or create a new group:").pack(side=tk.LEFT, padx=5)
        group_entry = tk.Entry(frame_dropdown)
        group_entry.pack(side=tk.LEFT, padx=5)


        root.mainloop()

# 输出笔记所在分组
if ENABLE_NOTE:
    if selected_group:
        word_group = ""
    else:
        word_group = ""
        if selected_note_file:
            if selected_note_file == DEFAULT_NOTE_FILE:
                pass
            else:
                word_group = "[" + selected_note_file.replace('GoldenDict-MyNote-', '').replace('.txt', '') + "]    "

# 输出 HTML 字体设置
if ENABLE_NOTE:
    result_temp = result
    
    # 输出 HTML 字体设置
    # result_temp = f'<span style="font-family: \'LXGW WenKai GB\', \'Simsun\';">{result_temp}</span>'

    # 输出 HTML 日语字体设置
    # result_temp = result_temp.replace('「', '<span style="font-family: \'Klee One\', \'Yu Gothic\', \'Yu Mincho\';">「').replace('」', '」</span>')
    # result_temp = result_temp.replace('『', '<span style="font-family: \'Klee One\', \'Yu Gothic\', \'Yu Mincho\';">『').replace('』', '』</span>')
    

# 输出 HTML 格式
if ENABLE_NOTE:
    if selected_group:
        pass
    else:
        if len_results < 2:    # 笔记未找到或仅见于一个分组
            output_html = f"""
            <html>
            <body>
            <p>{query_output}</p>
            <p>{result_temp}</p>
            <p>{word_group}<a href="{word}?edit=true" id="edit_link">Edit my note</a></p>
            </body>
            </html>
            """
        else:   # 笔记见于多个分组
            links = ", ".join(
                f'<a href="{word}={word_group}?edit=true" id="edit_link">[{word_group}]</a>'
                for word_group in all_group_names
            )

            output_html = f"""
            <html>
            <body>
            <p>{query_output}</p>
            <p>{result_temp}</p>
            <p>Edit my note in group:
            {links}</p>
            </body>
            </html>
            """
else:
    output_html = f"""
    <html>
    <body>
    <p>{query_output}</p>
    </body>
    </html>
    """


# 如果查询词包含 ?edit=true，启动 Tkinter 编辑窗口
if is_edit_mode and ENABLE_NOTE:
    edit_interface()
else:
    # 输出 HTML 内容
    print(output_html)
